/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package cz.muni.fi.pv168.gravemanager.backend;

import java.sql.*;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * This class implements GraveManager.
 * 
 * @author Petr Adamek
 */
public class GraveManagerImpl implements GraveManager {
    public static final Logger logger = Logger.getLogger(GraveManagerImpl.class.getName());

    public GraveManagerImpl(Connection conn) {
        this.conn = conn;
    }    
    
    // POZOR! Toto je jenom jednoducha ukazka, ktera neni vlaknove bezpecna a 
    // neresi transakce. Spravne reseni s transakcemi a pouzitim DataSource 
    // bude v prikladu k dalsi uloze    
    private Connection conn;
    
    @Override
    public void createGrave(Grave grave) throws ServiceFailureException {
        
        if (grave == null) {
            throw new IllegalArgumentException("grave is null");            
        }
        if (grave.getId() != null) {
            throw new IllegalArgumentException("grave id is already set");            
        }
        if (grave.getColumn() < 0) {
            throw new IllegalArgumentException("grave column is negative number");            
        }
        if (grave.getRow() < 0) {
            throw new IllegalArgumentException("grave row is negative number");            
        }
        if (grave.getCapacity() <= 0) {
            throw new IllegalArgumentException("grave column is negative number");            
        }

        PreparedStatement st = null;
        try {
            st = conn.prepareStatement(
                    "INSERT INTO GRAVES (gravecolumn,graverow,capacity,note) VALUES (?,?,?,?)",
                    Statement.RETURN_GENERATED_KEYS);
            st.setInt(1, grave.getColumn());
            st.setInt(2, grave.getRow());
            st.setInt(3, grave.getCapacity());
            st.setString(4, grave.getNote());
            int addedRows = st.executeUpdate();
            if (addedRows != 1) {
                throw new ServiceFailureException("Internal Error: More rows "
                        + "inserted when trying to insert grave " + grave);
            }            
            
            ResultSet keyRS = st.getGeneratedKeys();
            grave.setId(getKey(keyRS,grave));
            
        } catch (SQLException ex) {
            throw new ServiceFailureException("Error when inserting grave " + grave, ex);
        } finally {
            if (st != null) {
                try {
                    st.close();
                } catch (SQLException ex) {
                    logger.log(Level.SEVERE, null, ex);
                }
            }
        }
    }

    private Long getKey(ResultSet keyRS, Grave grave) throws ServiceFailureException, SQLException {
        if (keyRS.next()) {
            if (keyRS.getMetaData().getColumnCount() != 1) {
                throw new ServiceFailureException("Internal Error: Generated key"
                        + "retriving failed when trying to insert grave " + grave
                        + " - wrong key fields count: " + keyRS.getMetaData().getColumnCount());
            }
            Long result = keyRS.getLong(1);
            if (keyRS.next()) {
                throw new ServiceFailureException("Internal Error: Generated key"
                        + "retriving failed when trying to insert grave " + grave
                        + " - more keys found");
            }
            return result;
        } else {
            throw new ServiceFailureException("Internal Error: Generated key"
                    + "retriving failed when trying to insert grave " + grave
                    + " - no key found");
        }
    }

    @Override
    public Grave getGrave(Long id) throws ServiceFailureException {
        PreparedStatement st = null;
        try {
            st = conn.prepareStatement(
                    "SELECT id,gravecolumn,graverow,capacity,note FROM graves WHERE id = ?");
            st.setLong(1, id);
            ResultSet rs = st.executeQuery();
            
            if (rs.next()) {
                Grave grave = resultSetToGrave(rs);

                if (rs.next()) {
                    throw new ServiceFailureException(
                            "Internal error: More entities with the same id found "
                            + "(source id: " + id + ", found " + grave + " and " + resultSetToGrave(rs));                    
                }            
                
                return grave;
            } else {
                return null;
            }
            
        } catch (SQLException ex) {
            throw new ServiceFailureException(
                    "Error when retrieving grave with id " + id, ex);
        } finally {
            if (st != null) {
                try {
                    st.close();
                } catch (SQLException ex) {
                    logger.log(Level.SEVERE, null, ex);
                }
            }
        }
    }

    private Grave resultSetToGrave(ResultSet rs) throws SQLException {
        Grave grave = new Grave();
        grave.setId(rs.getLong("id"));
        grave.setColumn(rs.getInt("gravecolumn"));
        grave.setRow(rs.getInt("graverow"));
        grave.setCapacity(rs.getInt("capacity"));
        grave.setNote(rs.getString("note"));
        return grave;
    }

    @Override
    public void updateGrave(Grave grave) throws ServiceFailureException {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public void deleteGrave(Grave grave) throws ServiceFailureException {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public List<Grave> findAllGraves() throws ServiceFailureException {
        PreparedStatement st = null;
        try {
            st = conn.prepareStatement(
                    "SELECT id,gravecolumn,graverow,capacity,note FROM graves");
            ResultSet rs = st.executeQuery();
            
            List<Grave> result = new ArrayList<Grave>();
            while (rs.next()) {
                result.add(resultSetToGrave(rs));
            }
            return result;
            
        } catch (SQLException ex) {
            throw new ServiceFailureException(
                    "Error when retrieving all graves", ex);
        } finally {
            if (st != null) {
                try {
                    st.close();
                } catch (SQLException ex) {
                    logger.log(Level.SEVERE, null, ex);
                }
            }
        }
    }

    public static void main(String[] args)
    {
        try {
            Class.forName("org.apache.derby.jdbc.ClientDriver");
            Connection con = DriverManager.getConnection("jdbc:derby://localhost:1527/jdbcdemo", "demo","demo");
            GraveManager manager = new GraveManagerImpl(con);
            Grave grave1 = new Grave(),
                  grave2 = new Grave();
            grave1.setColumn(1);
            grave1.setRow(1);
            grave1.setNote("Spi sladce");
            grave1.setCapacity(3);
            grave2.setColumn(1);
            grave2.setRow(2);
            grave2.setNote("In loving memory");
            grave2.setCapacity(3);
            manager.createGrave(grave2);
            manager.createGrave(grave1);
            
            List<Grave> graves = manager.findAllGraves();
            System.out.println("Graves:");
            for(Grave grave:graves)
            {
                System.out.println(grave);
            }
            
            System.out.println(manager.getGrave(10L));
            
            
        } catch (SQLException ex) {
            logger.log(Level.SEVERE, "Chyba pri navazovani spojeni", ex);
        } catch (ClassNotFoundException ex) {
            logger.log(Level.SEVERE, "Nelze najit driver.", ex);
        }
        
        
    }
}
